### this R-language program code is for Chen (2013). 
### Function PLS_thre() is to estimate the threshold value by profiled least square
### Function FGLS_thre() is to estimate the threshold value by FGlS method
### bpwald() will generate bootstrapped critical values for the sup-wald test.
### bpwald1() is a function which will be used in bpwald().


library(tseries) 
library(MASS)
library(sandwich)
library(Hmisc);
library(timeDate);
library(timeSeries);
library(fBasics);


moment<-function(x){ 
  x=as.matrix(x);
  y=t(x)%*%x;
  return(y);
}

prod<-function(x,y){
  x=as.matrix(x);
  kk=dim(x)[2];
  z=x*matrix(rep(y,kk),ncol=kk)
  return(z);
}


PLS_thre<-function(dat,yi,xi,qi,h=0,intercept=0){

#  dat     = data matrix (nxk)
# yi      = index of dependent (y) variable, e.g.: yi = 1
# xi      = indexes of independent (x) variables, e.g.: xi = 2|3
# qi      = index of threshold (q) variable, e.g.: qi = 4;
# h       = heteroskedasticity indicator
#         Set h=0 to impose homoskedasticity assumption
#         Set h=1 to use White-correction for heteroskedasticity
#  qhat    = LS estimate of threshold

graph   = 1;     # Set _graph=0 to not view the graph of the wald test stat
conf1 = .95;  #Confidence Level for Confidence Regions  
     
n = dim(dat)[1];
q = dat[,qi];
y = dat[,yi];

if(intercept==1)   # include the intercept into the regression or not 
{
x = cbind(1,dat[,xi]);
 }else{
x = as.matrix(dat[,xi]);
}

k = dim(x)[2];

#### linear model estimation

mi = solve(moment(x));
fm=lm(y~x-1);
alpha = mi%*%(t(x)%*%y);
e=y-x%*%alpha;
ee=t(e)%*%e;
sig = ee/(n-k);
xe = prod(x,e);
if(h==0){
  se = sqrt(diag(mi)%*%sig);
}else{
  se = sqrt(diag(vcovHAC(fm)));
 }
vy = sum((y - mean(y))^2);
r_2 = 1-ee/vy;

linalpha=cbind(alpha,se);
colnames(linalpha)=c("est","se")

#### threshold model estimation
gn=200         ### grid size
trim=0.15      ### trim number at the beginning and the end
qq=sort(q); 
gamma1=seq(qq[round(trim*length(qq))],to=qq[round((1-trim)*length(qq))],length.out=gn);
qs=gamma1;

qn =length(qs);
sn = rep(0,qn);
store=sn;
z0=as.matrix(x);
r=1; 
while(r<=qn){
  ## calculate wald_stat
    	d1=(q<= qs[r])*1;
    	d2=1-d1;    
    	z1=prod(z0,d1);
    	z2=prod(z0,d2);
    	zz=cbind(z1,z2);
   	m=ginv(moment(zz));
    	theta=ginv(moment(zz))%*%(t(zz)%*%y);
    	e=y-zz%*%theta;
      sn[r]=t(e)%*%e;
  	ze=prod(zz,e);
  	v=moment(ze);
  	I=diag(rep(1,k));
  	R=cbind(I,-I);
  	w=t(R%*%theta)%*%ginv(R%*%m%*%v%*%m%*%t(R))%*%R%*%theta;
    	store[r]=w;
       r=r+1;
}

### sup_wald test statistic and sup_LM test statistic

rmin=which.min(sn);
sup_wald=max(store);
smin=sn[rmin];
sighat=smin/(n-2*k);
if(graph==1){ plot(qs,store,type="l");
   title("wald statistics for each threshold value");
  }

### LS estimation of the threshold value
 
qhat=qs[rmin];

#### confidence intervals for the threshold estimation

cc1=which(q<= qhat);
cc2=which(q> qhat);
x1 = x[cc1,];
y1 = y[cc1];
x2 = x[cc2,];
y2 = y[cc2];

mi1 = solve(moment(x1));
mi2 = solve(moment(x2));

fm1=lm(y1~x1-1);
fm2=lm(y2~x2-1);

alpha1 = as.matrix(coef(fm1));
alpha2 = as.matrix(coef(fm2));

e1 = y1 - x1%*%alpha1;
e2 = y2 - x2%*%alpha2;
ej=rep(0,length(q));

ej[cc1]=e1;
ej[cc2]=e2;
n1 = dim(y1)[1];
n2 = dim(y2)[1];
ee1 = t(e1)%*%e1;
ee2 = t(e2)%*%e2; 
sig1 = ee1/(n1-k);
sig2 = ee2/(n2-k);
sig_jt = (ee1+ee2)/(n-k*2);

if(h==0){
  	se1 = sqrt(diag(mi1)%*%sig_jt);
	se2 = sqrt(diag(mi2)%*%sig_jt);
 }else{
	  se1 = sqrt(diag(vcovHAC(fm1)));
	  se2 = sqrt(diag(vcovHAC(fm2)));
	  }
### caclulare the R_square
	vy1 = sum((y1 - mean(y1))^2);
	vy2 = sum((y2 - mean(y2))^2);
	r2_1 = 1 - ee1/vy1;
	r2_2 = 1 - ee2/vy2;
	r2_joint = 1 - (ee1+ee2)/vy;
	if(h==0){
	  lr  = (sn-smin)/sighat;
	}else{
	  r1 = (x%*%(alpha1-alpha2))^2;
	  r2 = r1*(ej^2);
	  qx = cbind(1,q,q^2);
	  qh = cbind(1,qhat,qhat^2);
        ### using regression
	  m1 = solve(moment(qx))%*%(t(qx)%*%r1);
	  m2 = solve(moment(qx))%*%(t(qx)%*%r2);
	  g1 = qh%*%m1;
	  g2 = qh%*%m2;      
        eta2 = g2/g1;
        lr = (sn-smin)/eta2;
  }
c1 = -2*log(1-sqrt(conf1));
lr1 = (lr >= c1);

if(max(lr1)==1){;
  qhat1 = qs[which.min(lr1)];
  qhat2 = qs[qn+1-which.min(rev(lr1))];
}else{
  qhat1 = qs[1];
  qhat2 = qs[qn];
}

## qhat1 is the low-bound of confidence interval and qhat2 is the up-bound.
threshold=c(qhat,qhat1,qhat2);
###### estimation and confidence intervals for beta1 and beta2;

z =which.max((pnorm(seq(.01,by=.01,length.out=200))*2-1) >= conf1)/100;

alpha1l = alpha1 - se1*z;
alpha1u = alpha1 + se1*z;
alpha2l = alpha2 - se2*z;
alpha2u = alpha2 + se2*z;

alpha1_hat=cbind(alpha1,se1,alpha1l,alpha1u)
colnames(alpha1_hat)=c("est","se","low_confidence","up_confidence")

alpha2_hat=cbind(alpha2,se2,alpha2l,alpha2u)
colnames(alpha2_hat)=c("est","se","low_confidence","up_confidence")

#### estimation and confidence intervals for delta;

d1=(q<= qhat)*1;
z1=prod(z0,d1);
zz=cbind(z0,z1);
mm=ginv(moment(zz));
fm3=lm(y~zz-1);
theta=as.matrix(coef(fm3));
e=y-zz%*%theta;
ee=t(e)%*%e; 
sig_jt = ee/(n-k*2);

if(h==0){
  	se = sqrt(diag(mm)%*%sig_jt);
 }else{
	se = sqrt(diag(vcovHAC(fm3)));
	 }
thetal = theta - se*z;
thetau = theta + se*z;

theta=cbind(theta,se,thetal,thetau);
colnames(theta)=c("est","se","low_confidence","up_confidence")
delta_hat=theta[(k+1):(2*k),];

result=list(walds=store,qt=qs,threshold=threshold,rss=ee1+ee2,alpha1=alpha1_hat,alpha2=alpha2_hat,delta=delta_hat,sup_wald=sup_wald);
return(result);
}

################################################################################
####  FGLS estimation of threshold models
 
FGLS_thre<-function(dat,yi,xi,qi,threshold,maxk=0,h=0,intercept=0){

#  dat     = data matrix (nxk)
# yi      = index of dependent (y) variable, e.g.: yi = 1
# xi      = indexes of independent (x) variables, e.g.: xi = 2|3
# qi      = index of threshold (q) variable, e.g.: qi = 4;
# h       = heteroskedasticity indicator
#         Set h=0 to impose homoskedasticity assumption
#         Set h=1 to use White-correction for heteroskedasticity
#  threshold  : primary estimation of the threshold value from the PLS method
#  maxk: number of leads and lags terms to deal with endogeneity, maxk=0, no leads and lags terms;


graph   = 1;     # Set _graph=0 to not view the graph of the wald test stat
                  
conf1 = .95;  #Confidence Level for Confidence Regions  


n = dim(dat)[1];
q = dat[,qi];
y = dat[,yi];
T=length(y);

if(intercept==1)
{
bx = cbind(1,dat[,xi]);
 }else{
bx = as.matrix(dat[,xi]);
}

k = dim(bx)[2];
x=as.matrix(bx[,1:(k-maxk)]);   ### integrated regressors
k1=k-maxk;   ###dimension for integrated regressors
k2=maxk;  ### dimension for leads and lags 
z=NULL;
if(k2>0){
  z=as.matrix(bx[,(k-maxk+1):k]);
}

q1=q[2:T];
y1=y[2:T];
x1=as.matrix(x[2:T,]);
y2=y[1:(T-1)];
x2=as.matrix(x[1:(T-1),]);
q2=q[1:(T-1)];

gn=200  ### grid size
trim=0.15  ### trim number at the beginning and the end

gn=200         ### grid size
trim=0.15      ### trim number at the beginning and the end
qq=sort(q); 
qs=seq(qq[round(trim*length(qq))],to=qq[round((1-trim)*length(qq))],length.out=gn);
qn =length(qs);

rou_hat=NULL;
sd1=10000;
step=1;
threshold1=threshold;
########################### 
## recursively search the threshold value

while(sd1>=0.01&step<=5){
      step=step+1;
      ### estimate rou
	d1=(q<= threshold1);
	d2=1-d1;
	xx1=prod(x,d1);
	xx2=prod(x,d2);
	bx1=cbind(xx1,xx2,z);
	fm1=lm(y~bx1-1);
	e_hat=residuals(fm1);
	fme2=lm(e_hat[2:length(e_hat)]~e_hat[1:(length(e_hat)-1)]-1);
	rou1=coef(fme2);

	if(k2>0){
      	 z_tuta=z[2:T,]-rou1*z[1:(T-1)];
        	}else{ z_tuta=NULL; }

	sn=store=rep(0,qn);
	j=1; 
	while(j<=qn){
		threshold=qs[j];
		I1=(q1<=threshold);
		I2=(q1>threshold);
		I3=(q2<=threshold);
		I4=(q2>threshold);
		y_tuta=y1-y2*rou1;
  	      x1_tuta=diag(I1)%*%x1-rou1*diag(I3)%*%x2;
		x2_tuta=diag(I2)%*%x1-rou1*diag(I4)%*%x2;
		x_tuta=cbind(x1_tuta,x2_tuta,z_tuta);
     		m=ginv(moment(x_tuta));
		theta=m%*%(t(x_tuta)%*%y_tuta);
      	e=y_tuta-x_tuta%*%theta;
      	ze=prod(x_tuta,e);
      	v=moment(ze);
      	I=diag(rep(1,k1));
       	R=cbind(I,-I);
      	if(k2>0){ 
       		 I1=matrix(0,nrow=k1,ncol=k2);
       		 R=cbind(R,I1);}
      	w=t(R%*%theta)%*%ginv(R%*%m%*%v%*%m%*%t(R))%*%R%*%theta;
      	store[j]=w;
      	sn[j]=sum(e^2);
      	j=j+1;
    	}
wald01=max(store);
j1=which.max(store);
j=which.min(sn);
smin=min(sn);
sighat=smin/(n-2*k1-k2);

##### confidence interval for the threshold value 
lr  = (sn-smin)/sighat;
c1 = -2*log(1-sqrt(conf1));
lr1 = (lr >= c1);
if(max(lr1)==1){;
  qhat1 = qs[which.min(lr1)];
  qhat2 = qs[qn+1-which.min(rev(lr1))];
}else{
  qhat1 = qs[1];
  qhat2 = qs[qn];
}

threshold=qs[j];
sd1=abs(threshold-threshold1);
d1=(q<= threshold);
d2=1-d1;
xx1=prod(x,d1);
xx2=prod(x,d2);
bx1=cbind(xx1,xx2,z);
fm1=lm(y~bx1-1);
e_hat=residuals(fm1);
fme2=lm(e_hat[2:length(e_hat)]~e_hat[1:(length(e_hat)-1)]-1);
rou2=coef(fme2);
rou_hat=cbind(rou_hat,rou2);
rou1=rou2;
threshold1=threshold;
}

#### estimate other coefficients using the FGLS estimation for the threshold value;

if(k2>0){
       z_tuta=z[2:T,]-rou2*z[1:(T-1)];
        }else{
   z_tuta=NULL;
}

if(graph==1){ plot(qs,store,type="l");
   title("wald statistics for each threshold value");
  }


###linear model estimation

y_tuta=(y1-y2*rou2)*(q1<=threshold)+(y1-y2*rou2)*(q1>threshold)
linx_tuta=x1-x2*rou2
linx_tuta=cbind(linx_tuta,z_tuta)
linfm1=lm(y_tuta~linx_tuta-1);
lmi1 = solve(moment(linx_tuta));
le1 =residuals(linfm1);
lee1 = t(le1)%*%le1;
n1=length(y_tuta);
linsig =lee1/(n1-dim(linx_tuta)[2]);

if(h==0){
  lse1 = sqrt(diag(lmi1)%*%linsig);
}else{
  lse1 = sqrt(diag(vcovHAC(linfm1)));
}
lintheta=coef(linfm1);
lintheta=cbind(lintheta,lse1);

#### estimation for alpha1 and alpha2
I1=(q1<=threshold);
I2=(q1>threshold);
I3=(q2<=threshold);
I4=(q2>threshold);

x1_tuta=diag(I1)%*%x1-rou2*diag(I3)%*%x2;
x2_tuta=diag(I2)%*%x1-rou2*diag(I4)%*%x2;
x_tuta=cbind(x1_tuta,x2_tuta,z_tuta);

fm1=lm(y_tuta~x_tuta-1);
mi1 = solve(moment(x_tuta));
e1 =residuals(fm1);
ee1 = t(e1)%*%e1;
n1=length(y_tuta);
sig1 = ee1/(n1-dim(x_tuta)[2]);
if(h==0){
  se1 = sqrt(diag(mi1)%*%sig1);
}else{
  se1 = sqrt(diag(vcovHAC(fm1)));
}
alpha=coef(fm1);
vy = sum((y_tuta - mean(y_tuta))^2);
r2_joint = 1 - ee1/vy;
alpha_hat=cbind(alpha,se1);
colnames(alpha_hat)=c("est","se")

alpha1_hat=alpha_hat[1:k1,];
alpha2_hat=alpha_hat[(k1+1):(2*k1),];


#### estimation of delta, 
x1_tuta1=x1-rou2*x2;
x2_tuta1=diag(I1)%*%x1-rou2*diag(I3)%*%x2;
x_tuta1=cbind(x1_tuta1,x2_tuta1,z_tuta);

fm1=lm(y_tuta~x_tuta1-1);
mi1 = solve(moment(x_tuta1));
e1 =residuals(fm1);
ee1 = t(e1)%*%e1;
n1=length(y_tuta);
sig1 = ee1/(n1-dim(x_tuta)[2]);
if(h==0){
  se1 = sqrt(diag(mi1)%*%sig1);
}else{
  se1 = sqrt(diag(vcovHAC(fm1)));
}
alpha=coef(fm1);
theta_hat=cbind(alpha,se1);
colnames(theta_hat)=c("est","se")
delta_hat=theta_hat[(k1+1):(2*k1),];
threshold=c(threshold,qhat1,qhat2);

result=list(walds=store,qt=qs,rou_hat=rou_hat,threshold=threshold,alpha1=alpha1_hat,alpha2=alpha2_hat,delta=delta_hat,sup_wald=wald01);
return(result);
}

##### bp wald test

bpwald<-function(dat,yi,xi,qi,threshold,rou1,maxk=0,intercept=0,BN=500){
n = dim(dat)[1];
q = dat[,qi];
y = dat[,yi];
if(intercept==1){
	bx = cbind(1,dat[,xi]);
 }else{
	bx = as.matrix(dat[,xi]);
}

k = dim(bx)[2];
x=as.matrix(bx[,1:(k-maxk)]);
k1=k-maxk;   ###dimension for nonstationary covariate
k2=maxk;  ###dimension for stationary covariates
z=NULL;
if(k2>0){
  z=as.matrix(bx[,(k-maxk+1):k]);
}
q1=q[2:n];
q2=q[1:(n-1)];
y1=y[2:n];
y2=y[1:(n-1)];
x1=as.matrix(x[2:n,]);
x2=as.matrix(x[1:(n-1),]);
z_tuta=NULL;
if(maxk>0){
	z=as.matrix(z);
	z1=as.matrix(z[2:n,]);
	z2=as.matrix(z[1:(n-1),]);
	z_tuta=z1-z2*rou1;
}

I1=(q1<=threshold);
I2=(q1>threshold);
I3=(q2<=threshold);
I4=(q2>threshold);

y_tuta=(y1-y2*rou1)*(q1<=threshold)+(y1-y2*rou1)*(q1>threshold)
linx_tuta=x1-x2*rou1;
#### linear model
linx_tuta=cbind(linx_tuta,z_tuta)
linfm1=lm(y_tuta~linx_tuta-1);
lmi1 = solve(moment(linx_tuta));
le1 =residuals(linfm1);
rss0= t(le1)%*%le1;
n1=length(y_tuta);
lintheta=coef(linfm1);
#### residuals
x1_tuta=diag(I1)%*%x1-rou1*diag(I3)%*%x2;
x2_tuta=diag(I2)%*%x1-rou1*diag(I4)%*%x2;
x_tuta=cbind(linx_tuta,x1_tuta);
fm1=lm(y_tuta~x_tuta-1);
e1 =residuals(fm1);
e2=tsbootstrap(e1,nb=BN);
e3=matrix(0,nrow=length(y),ncol=BN);
e3[1,]=rnorm(BN);
for(j in 2:length(y)){
  e3[j,]=rou1*e3[(j-1),]+e2[j-1,];
}
clm1=clm2=cwald1=cwald2=rep(0,BN);
for(bn in 1:BN){
      yb=bx%*%as.matrix(lintheta)+e3[,bn];
	dat1=cbind(yb,bx,q);
	yi1=1;
	xi1=2:(ncol(dat1)-1);
	qi1=ncol(dat1);
	thre=bpwald1(dat1,1,xi1,qi1,rou1,maxk);
	bsuper_wald1=thre$sup_wald1
	bsuper_wald2=thre$sup_wald2
	cwald1[bn]=bsuper_wald1;
	cwald2[bn]=bsuper_wald2;
	}
   cvwald1=rep(0,6);
   cvwald1=sort(cwald1)[c(BN*0.01,BN*0.05,BN*0.1,BN*0.90,BN*0.95,BN*0.99)];
   cvwald2=rep(0,6);
   cvwald2=sort(cwald2)[c(BN*0.01,BN*0.05,BN*0.1,BN*0.90,BN*0.95,BN*0.99)];
   result=list(cvwald1=cvwald1,cvwald2=cvwald2);
  return(result);
}

bpwald1<-function(dat,yi,xi,qi,rou1,maxk){
n = dim(dat)[1];
q = dat[,qi];
y = dat[,yi];
T=length(y);
bx = as.matrix(dat[,xi]);

k = dim(bx)[2];
x=as.matrix(bx[,1:(k-maxk)]);
k1=k-maxk;   ###dimension for nonstationary covariate
k2=maxk;  ###dimension for stationary covariates
z_tuta=z=NULL;
if(k2>0){
      z=as.matrix(bx[,(k-maxk+1):k]);
      z_tuta=z[2:T,]-rou1*z[1:(T-1)];
    	}
q1=q[2:T];
y1=y[2:T];
x1=as.matrix(x[2:T,]);
y2=y[1:(T-1)];
x2=as.matrix(x[1:(T-1),]);
q2=q[1:(T-1)];

#### linear model estimation
fm=lm(y~x-1);
e=residuals(fm);
rss0=t(e)%*%e;

gn=200  ### grid size
trim=0.15  ### trim number at the beginning and the end
qq=sort(q); 
qs=seq(qq[round(trim*length(qq))],to=qq[round((1-trim)*length(qq))],length.out=gn);
qn =length(qs);

sn1=sn2=store1=store2=rep(0,qn);
j=1; 
while(j<=qn){
    	d1=(q<= qs[j])*1;
    	d2=1-d1;    
    	xx1=prod(x,d1);
    	xx2=prod(x,d2);
    	xx=cbind(xx1,xx2);
    	m=ginv(moment(xx));
    	theta=ginv(moment(xx))%*%(t(xx)%*%y);
    	e=y-xx%*%theta;
    	sn1[j]=t(e)%*%e;
    	ze=prod(xx,e);
    	v=moment(ze);
    	I=diag(rep(1,k1));
    	R=cbind(I,-I);
    	w1=t(R%*%theta)%*%ginv(R%*%m%*%v%*%m%*%t(R))%*%R%*%theta;
    	store1[j]=w1;
      #### wald for FGLS
    	threshold=qs[j];
    	I1=(q1<=threshold);
    	I2=(q1>threshold);
    	I3=(q2<=threshold);
	I4=(q2>threshold);
	y_tuta=y1-y2*rou1;
  	x1_tuta=diag(I1)%*%x1-rou1*diag(I3)%*%x2;
	x2_tuta=diag(I2)%*%x1-rou1*diag(I4)%*%x2;
	x_tuta=cbind(x1_tuta,x2_tuta,z_tuta);
     	m=ginv(moment(x_tuta));
	theta2=m%*%(t(x_tuta)%*%y_tuta);
      e2=y_tuta-x_tuta%*%theta2;
      ze2=prod(x_tuta,e2);
      v=moment(ze2);
      I=diag(rep(1,k1));
      R=cbind(I,-I);
      if(k2>0){ 
       	I1=matrix(0,nrow=k1,ncol=k2);
       	R=cbind(R,I1);}
      w2=t(R%*%theta2)%*%ginv(R%*%m%*%v%*%m%*%t(R))%*%R%*%theta2;
      store2[j]=w2;
      sn2[j]=sum(e2^2);
      j=j+1;
    	}
supwald1=max(store1);
supwald2=max(store2);

result=list(sup_wald1=supwald1,sup_wald2=supwald2);
return(result);
}







