# Experiment 2 for Chen (2013)
#### size performance

N=1000  # repeat times
n1=400;  # sample size, could be 100, 200, 400;
# parameters set up
para=rbind(
c(n1,1,1,0,0.1,0),
c(n1,1,1,0,0.5,0),
c(n1,1,1,0,0.1,0.5),
c(n1,1,1,0,0.5,0.5),
c(n1,0.95,0.95,0,0.1,0),
c(n1,0.95,0.95,0,0.5,0),
c(n1,0.95,0.95,0,0.1,0.5),
c(n1,0.95,0.95,0,0.5,0.5),
c(n1,-0.95,-0.95,0,0.1,0),
c(n1,-0.95,-0.95,0,0.5,0),
c(n1,-0.95,-0.95,0,0.1,0.5),
c(n1,-0.95,-0.95,0,0.5,0.5),
c(n1,0,0,0,0.1,0),
c(n1,0,0,0,0.5,0),
c(n1,0,0,0,0.1,0.5),
c(n1,0,0,0,0.5,0.5)
)

dimp=dim(para)[1]; 
for(pp in 1:dimp)
{
   T=para[pp,1];  # sample size
   rou1=para[pp,2];  #persistency parameter in regime 1
   rou2=para[pp,3];  # persistency parameter in regime 2. Here, rou1 is set as the same with rou2, but could be different;
   beta1=0*n1^(-0.5-para[pp,5]);   # size of threshold effect, 0 to check the size performance
   beta2=para[pp,4];  
   rou=para[pp,6];  # endogeneity parameter

# data generating process
#define the threshold variable q as an AR(1) process
q=rep(0,T);
ee=rnorm(T)
for(j in 2:T){
 q[j]=0.5*q[j-1]+ee[j];
 }
gama0=0  ;  # true threshold value

#define error term process 
for(n2 in 1:N){
e=rnorm(T);
e1=rep(e[1],T);
for(i in 2:T){
  e1[i]=e1[i-1]*rou1*(q[i]<gama0)+e1[i-1]*rou2*(q[i]>=gama0)+e[i];
}
ex=rnorm(T);
x=cumsum(ex);
maxk=5;
bigex=NULL;
for(i in 1:maxk){
bigex=cbind(bigex,ex[i:(T-maxk+i)])
}
ex1=rep(0,T);
ex1[maxk:T]=bigex[,maxk];
y=(beta1*x+rou*ex1+e1)*(q<=gama0)+(beta2*x+rou*ex1+e1)*(q>gama0)

###
dat=cbind(y,x,q);
yi=1;
xi=2:(ncol(dat)-1);
qi=ncol(dat);
dat1=cbind(y[maxk:T],x[maxk:T],bigex,q[maxk:T]);
yi=1;
xi1=2:(ncol(dat1)-1);
qi1=ncol(dat1);

swald1=swald2=matrix(0,nrow=N,ncol=dimp)
cvwald1=cvwald2=matrix(0,nrow=6,ncol=dimp);
thre1=PLS_thre(dat,1,xi,qi,h=1,intercept=0);   #primary estimation of threshold
threshold=thre1$threshold[1];
thre2=FGLS_thre(dat1,1,xi1,qi1,maxk=maxk,threshold,h=1,intercept=0);   #primary estimation of threshold, but with stationary terms
swald1[n2,pp]=thre1$sup_wald
swald2[n2,pp]=thre2$sup_wald;
    }
rou_hat=thre2$rou_hat;
rou2=rou_hat[nrow(rou_hat),1];
threshold=thre2$threshold[1];
cv=bpwald(dat1,1,xi1,qi1,threshold,rou2,maxk=maxk,BN=500);
cvwald1[,pp]=cv$cvwald1;
cvwald2[,pp]=cv$cvwald2;
}

#### power calculation
power_wald1=power_wald2=matrix(0,nrow=3,ncol=dimp);
for(pp in 1:dimp){
power_wald1[1,pp]=sum((swald1[,pp]>cvwald1[4,pp]))/N;
power_wald1[2,pp]=sum((swald1[,pp]>cvwald1[5,pp]))/N;
power_wald1[3,pp]=sum((swald1[,pp]>cvwald1[6,pp]))/N;
power_wald2[1,pp]=sum((swald2[,pp]>cvwald2[4,pp]))/N;
power_wald2[2,pp]=sum((swald2[,pp]>cvwald2[5,pp]))/N;
power_wald2[3,pp]=sum((swald2[,pp]>cvwald2[6,pp]))/N;
}

cbind(para,t(power_wald1))
cbind(para,t(power_wald2))
pr1=pr2=rep(0,8);  # model selection for weak identification case
for(pp in 1:8){
pr1[pp]=sum(swald1[,pp]>log(n1)^2)/N;
pr2[pp]=sum(swald2[,pp]>log(n1)^2)/N
}
save.image(file="size_exp2_t400_N1000.rdata")


### Power performance

N=1000  # repeat times
n1=400;  # sample size, could be 100, 200, 400;
# parameters set up
para=rbind(
c(n1,1,1,0,0.1,0),
c(n1,1,1,0,0.5,0),
c(n1,1,1,0,0.1,0.5),
c(n1,1,1,0,0.5,0.5),
c(n1,0.95,0.95,0,0.1,0),
c(n1,0.95,0.95,0,0.5,0),
c(n1,0.95,0.95,0,0.1,0.5),
c(n1,0.95,0.95,0,0.5,0.5),
c(n1,-0.95,-0.95,0,0.1,0),
c(n1,-0.95,-0.95,0,0.5,0),
c(n1,-0.95,-0.95,0,0.1,0.5),
c(n1,-0.95,-0.95,0,0.5,0.5),
c(n1,0,0,0,0.1,0),
c(n1,0,0,0,0.5,0),
c(n1,0,0,0,0.1,0.5),
c(n1,0,0,0,0.5,0.5)
)

dimp=dim(para)[1];

for(pp in 1:dimp)
{
   T=para[pp,1];  # sample size
   rou1=para[pp,2];  #persistency parameter in regime 1
   rou2=para[pp,3];  # persistency parameter in regime 2
   beta1=4*n1^(-0.5-para[pp,5]);   ### size of threshold effect, for power
   beta2=para[pp,4];  
   rou=para[pp,6];  # endogeneity parameter

# data generating process
#define the threshold variable q as an AR(1) process
q=rep(0,T);
ee=rnorm(T)
for(j in 2:T){
 q[j]=0.5*q[j-1]+ee[j];
 }
gama0=0  ;  # true threshold value

#define error term process 
for(n2 in 1:N){
e=rnorm(T);
e1=rep(e[1],T);
for(i in 2:T){
  e1[i]=e1[i-1]*rou1*(q[i]<gama0)+e1[i-1]*rou2*(q[i]>=gama0)+e[i];
}
ex=rnorm(T);
x=cumsum(ex);
maxk=5;
bigex=NULL;
for(i in 1:maxk){
bigex=cbind(bigex,ex[i:(T-maxk+i)])
}
ex1=rep(0,T);
ex1[maxk:T]=bigex[,maxk];
y=(beta1*x+rou*ex1+e1)*(q<=gama0)+(beta2*x+rou*ex1+e1)*(q>gama0)

###
dat=cbind(y,x,q);
yi=1;
xi=2:(ncol(dat)-1);
qi=ncol(dat);
dat1=cbind(y[maxk:T],x[maxk:T],bigex,q[maxk:T]);
yi=1;
xi1=2:(ncol(dat1)-1);
qi1=ncol(dat1);

swald1=swald2=matrix(0,nrow=N,ncol=dimp)
cvwald1=cvwald2=matrix(0,nrow=6,ncol=dimp);
thre1=PLS_thre(dat,1,xi,qi,h=1,intercept=0);   #primary estimation of threshold
threshold=thre1$threshold[1];
thre2=FGLS_thre(dat1,1,xi1,qi1,maxk=maxk,threshold,h=1,intercept=0);   #primary estimation of threshold, but with stationary terms
swald1[n2,pp]=thre1$sup_wald
swald2[n2,pp]=thre2$sup_wald;
    }
rou_hat=thre2$rou_hat;
rou2=rou_hat[nrow(rou_hat),1];
threshold=thre2$threshold;
cv=bpwald(dat1,1,xi1,qi1,threshold,rou2,maxk=maxk,BN=500);
cvwald1[,pp]=cv$cvwald1;
cvwald2[,pp]=cv$cvwald2;
}

#### power calculation
power_wald1=power_wald2=matrix(0,nrow=3,ncol=dimp);
for(pp in 1:dimp){
power_wald1[1,pp]=sum((swald1[,pp]>cvwald1[4,pp]))/N;
power_wald1[2,pp]=sum((swald1[,pp]>cvwald1[5,pp]))/N;
power_wald1[3,pp]=sum((swald1[,pp]>cvwald1[6,pp]))/N;
power_wald2[1,pp]=sum((swald2[,pp]>cvwald2[4,pp]))/N;
power_wald2[2,pp]=sum((swald2[,pp]>cvwald2[5,pp]))/N;
power_wald2[3,pp]=sum((swald2[,pp]>cvwald2[6,pp]))/N;
}

cbind(para,t(power_wald1))
cbind(para,t(power_wald2))
pr1=pr2=rep(0,8);  # model selection for weak identification case
for(pp in 1:8){
pr1[pp]=sum(swald1[,pp]>log(n1)^2)/N;
pr2[pp]=sum(swald2[,pp]>log(n1)^2)/N
}

save.image(file="power_exp2_t400_N1000.rdata")


